'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const util = require("util");
const chalk_1 = require("chalk");
const MAX_RESOURCES_PER_CLOUDFORMATION_STACK = 500;
class SlsCloudformationResourceCounter {
    constructor(serverless, options) {
        this.serverless = serverless;
        this.options = options;
        this.hooks = {
            'after:deploy:deploy': this.hookWrapper.bind(this, this.countAwsResources),
        };
    }
    /**
     * Wrapper for lifecycle function, initializes variables.
     * @param lifecycleFunc lifecycle function that actually does desired action
     */
    hookWrapper(lifecycleFunc) {
        return __awaiter(this, void 0, void 0, function* () {
            this.initializeVariables();
            return yield lifecycleFunc.call(this);
        });
    }
    /**
     * Counts the generated AWS resources.
     */
    countAwsResources() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const resources = yield this.getCreatedAwsResources();
                this.displayResourceCount(resources.length);
            }
            catch (error) {
                this.serverless.cli.log(util.format('Cannot count: %s!', error.message));
            }
        });
    }
    /**
     * Displays a given count as a number of AWS resources created.
     * Optionally, displays a warning if provided threshold is met or exceeded.
     * @param count resource count to display
     */
    displayResourceCount(count) {
        const limitMessage = `AWS CloudFormation has a hard limit of ${MAX_RESOURCES_PER_CLOUDFORMATION_STACK} resources for a deployed stack!`;
        let message = util.format('CloudFormation resources count: %d', count);
        if (this.warningThreshold && count >= this.warningThreshold) {
            message += `\n${chalk_1.default.red('WARNING:')}\n`;
            message += `${chalk_1.default.red(limitMessage)}\n`;
        }
        this.serverless.cli.log(message);
    }
    /**
     *  Gets a listing of created AWS resources
     */
    getCreatedAwsResources() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackResources = [];
            let result = yield this.cloudformation
                .listStackResources({ StackName: this.stackName })
                .promise();
            result.StackResourceSummaries.forEach((stackItem) => {
                stackResources.push(stackItem);
            });
            let morePages = result.NextToken ? true : false;
            while (morePages) {
                const request = {
                    NextToken: result.NextToken,
                    StackName: this.stackName,
                };
                result = yield this.cloudformation.listStackResources(request).promise();
                result.StackResourceSummaries.forEach((stackItem) => {
                    stackResources.push(stackItem);
                });
                morePages = result.NextToken ? true : false;
            }
            return Promise.all(stackResources);
        });
    }
    /**
     * Initializes plugin's variables
     */
    initializeVariables() {
        if (this.serverless.service.custom) {
            this.warningThreshold = this.serverless.service.custom.warningThreshold;
        }
        const credentials = this.serverless.providers.aws.getCredentials();
        credentials.region = this.serverless.providers.aws.getRegion();
        this.cloudformation = new this.serverless.providers.aws.sdk.CloudFormation(credentials);
    }
    /**
     * Returns the specified stack name or the default one otherwise.
     * @returns the stack name
     */
    get stackName() {
        const stackName = this.serverless.service.provider.stackName;
        return (stackName || util.format('%s-%s', this.serverless.service.service, this.serverless.service.provider.stage));
    }
}
module.exports = SlsCloudformationResourceCounter;
